#ifndef MMAP_DEV_HEAP_H
#define MMAP_DEV_HEAP_H

#ifdef MAP_DEV_HEAP
#include <fcntl.h>  /* for O_RDWR */
#include <unistd.h> /* for close  */
#define MMAP_FLAGS           (MAP_PRIVATE)

#include "malloc_utils.h"

extern int dev_heap_works;

static inline void* CALL_MMAP(size_t s)
{
  if (dev_heap_works) {
    /* Note that we have to open and close the file descriptor each
     * time because some processes may close all of their file descriptors.
     * For example, pulseaudio closes all of its file descriptors on startup
     * when dropping permissions. It's also common for processes to close
     * all file descriptors in the child process after forking (which may
     * occur after any pthread_atfork() handler, so we can't rely on that).
     * We don't want to try just re-opening a file descriptor after an mmap
     * fails because we may inadvertently mmap a real file.
     */
    int dev_heap_fd = open("/dev/heap", O_RDWR);
    
    if (dev_heap_fd >= 0) {
      void *ret = mmap(0, s, PROT_READ|PROT_WRITE, MMAP_FLAGS, dev_heap_fd, 0);
      close(dev_heap_fd);
      return ret;
    } else {
      dev_heap_works = 0;
    }
  }

  /* We couldn't open /dev/heap, so fall back to anonymous mmap */
  return mmap(0, s, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
}

#endif /* MAP_DEV_HEAP */
#endif /* MMAP_DEV_HEAP_H */
